﻿/**
* Sample code for the GetRates Canada Post service.
* 
* The GetRates service returns a list of shipping services, prices and transit times 
* for a given item to be shipped. 
*
* This sample is configured to access the Developer Program sandbox environment. 
* Use your development key username and password for the web service credentials.
* 
**/

using System;
using System.ServiceModel.Channels;
using System.ServiceModel;
using System.Configuration;
using System.Security.Authentication;
using System.Net;

namespace GetRates
{
    class GetRates
    {
        //TLS 1.2 support for .NET 3.5
        public const SslProtocols _Tls12 = (SslProtocols)0x00000C00;
        public const SecurityProtocolType Tls12 = (SecurityProtocolType)_Tls12;

        static void Main(string[] args)
        {
            ServicePointManager.SecurityProtocol = Tls12;

            // Your username, password and customer number are imported from the following file
            // CPCWS_SOAP_Rating_DotNet_Samples\SOAP\rating\user.xml 
            var username = ConfigurationSettings.AppSettings["username"];
            var password = ConfigurationSettings.AppSettings["password"];
            var mailedBy = ConfigurationSettings.AppSettings["customerNumber"];

            RatingPortTypeClient server = new RatingPortTypeClient("RatingPort", "https://ct.soa-gw.canadapost.ca/rs/soap/rating/v4");
            server.ClientCredentials.UserName.UserName = username;
            server.ClientCredentials.UserName.Password = password;

            // Create Request
            getratesrequest request = new getratesrequest();
            request.locale = "FR";

            request.mailingscenario = new getratesrequestMailingscenario();
            request.mailingscenario.parcelcharacteristics = new getratesrequestMailingscenarioParcelcharacteristics();
            request.mailingscenario.destination = new getratesrequestMailingscenarioDestination();
            getratesrequestMailingscenarioDestinationDomestic destDom = new getratesrequestMailingscenarioDestinationDomestic();
            request.mailingscenario.destination.Item = destDom;

            request.mailingscenario.customernumber = mailedBy;
            request.mailingscenario.parcelcharacteristics.weight = 1;
            request.mailingscenario.originpostalcode = "K2B8J6";
            destDom.postalcode = "J0E1X0";

            // Disable timestamp on request
            BindingElementCollection elements = server.Endpoint.Binding.CreateBindingElements();
            elements.Find<SecurityBindingElement>().IncludeTimestamp = false;
            server.Endpoint.Binding = new CustomBinding(elements);

            String responseAsString = "";

            try
            {
                // Execute Request
                getratesresponse response = server.GetRates(request);

                // Retrieve values from response object
                if (response.Item.GetType() == typeof(getratesresponsePricequotes))
                {
                    getratesresponsePricequotes priceQuotes = (getratesresponsePricequotes)response.Item;
                    foreach (var priceQuote in priceQuotes.pricequote)
                    {
                        responseAsString += "Service Name: " + priceQuote.servicename + "\r\n";
                        responseAsString += "Price Name: $" + priceQuote.pricedetails.due + "\r\n\r\n";
                    }
                }
                else
                {
                    messages msgs = (messages)response.Item;
                    foreach (var item in msgs.message)
                    {
                        responseAsString += "Error Code: " + item.code + "\r\n";
                        responseAsString += "Error Msg: " + item.description + "\r\n";
                    }
                }
            }
            catch (FaultException faultEx)
            {
                // SOAP Fault
                responseAsString += "Fault Code: " + faultEx.Code.Name + "\r\n";
                responseAsString += "Fault String: " + faultEx.Message + "\r\n";
            }
            catch (Exception ex)
            {
                // Misc Exception
                responseAsString += "ERROR: " + ex.Message + "\r\n";
            }

            Console.WriteLine(responseAsString);
            Console.WriteLine("Press any key to continue...");
            Console.ReadKey(true);

        }
    }
}
